@@ -1,7 +1,6 @@
 from csv import writer as csv_writer
 from datetime import datetime
 from enum import Enum
-from io import BytesIO
 from json import dumps
 from json import load
 from os.path import getsize
@@ -61,6 +60,7 @@ from .util import CompleteChoice
 from .util import CustomHelpColorsGroup
 from .util import EnvVars
 from .util import add_history
+from .util import backup_database
 from .util import clean_string
 from .util import color_option
 from .util import database_callback
@@ -155,7 +155,7 @@ def serializer(obj: object) -> object:
 def date_callback(ctx: Context, param: Parameter, value: str) -> tuple[str | None, ...] | None:
     if not value:
         return None
-    elif m := match(r"(?:(\d{4})(?:-(\d\d?)(?:-(\d\d?))?)?)?[ ]?(?:(\d\d?)(?::(\d\d?)(?::(\d\d?))?)?)?", value):
+    elif m := match(r"(?:(\d{4})(?:-(\d\d?)(?:-(\d\d?))?)?)? ?(?:(\d\d?)(?::(\d\d?)(?::(\d\d?))?)?)?", value):
         return m.groups()
     raise BadParameter("Not a valid date format.", ctx, param)
 
@@ -284,8 +284,9 @@ def search(table: Table, headers: list[str | Column], query: str, sort: tuple[tu
            offset: int = 0, sql: bool = False) -> tuple[Cursor, tuple[str, list[str]]]:
     cols_table: list[str] = [c.name for c in table.columns]
     query_elems, values = ([query], []) if sql or not query.strip() else query_to_sql(
-        query, "any", [*map(str.lower, {*cols_table, "any"} - {"ID", "AUTHOR", "USERNAME"})],
-        {"author": "replace(author, '_', '')", "any": f"({'||'.join(cols_table)})"})
+        query, "any", [*map(str.lower, {*cols_table, "any"} - {"ID", "FILESAVED", "USERUPDATE", "ACTIVE"})],
+        {"author": "replace(author, '_', '')",
+         "any": f"({'||'.join(set(cols_table) - {'FAVORITE', 'FILESAVED', 'USERUPDATE', 'ACTIVE'})})"})
     query = " ".join(query_elems)
     return (table.select_sql(query, values, headers, [" ".join(s) for s in sort], limit or 0, offset or 0),
             (query, values))
@@ -506,7 +507,7 @@ def database_info(ctx: Context, database: Callable[..., Database]):
         help=f"Show entries containing {yellow}FILTER{reset}.")
 @option("--filter-date", metavar="DATE", type=str, default=None, callback=date_callback,
         help=f"Filter by {yellow}DATE{reset}.")
-@option("--clear", is_flag=True, default=False, required=False, help="Clear entries.")
+@option("--clear", is_flag=True, default=False, help="Clear entries.")
 @database_exists_option
 @color_option
 @help_option
@@ -548,6 +549,7 @@ def database_history(ctx: Context, database: Callable[..., Database], clear: boo
 
     if clear:
         echo(f"Removed {yellow}{removed}{reset} entries.")
+        backup_database(db, ctx, "database")
 
 
 @database_app.command("search", short_help="Search database entries.", no_args_is_help=True)
@@ -617,19 +619,22 @@ def database_search(ctx: Context, database: Callable[..., Database], table: str,
 
     All other strings are considered search terms.
 
-    The search uses the {c}@any{r} field by default, allowing to do general searches without specifying a field.
+    The search uses the {c}@any{r} field by default for submissions and journals, allowing to do general searches
+    without specifying a field. The {c}@any{r} field does not include the {c}FAVORITE{r}, {c}FILESAVED{r},
+    {c}USERUPDATE{r}, and {c}ACTIVE{r} fields and must be searched manually using the respective query fields. When
+    searching users, {c}@username{r} is the default field.
 
     Search terms that are not separated by a logic operator are considered {i}AND{r} terms (i.e. {c}a b c{r} <->
     {c}a & b & c{r}).
 
-    Except for the {c}ID{r}, {c}AUTHOR{r}, and {c}USERNAME{r} fields, all search terms are matched by fields containing
-    the term: i.e. {c}@description cat{r} will match any item whose description field contains "cat". To match items
-    that contain only "cat" (or start with, end with, etc.), the {c}%{r}, {c}_{r}, {c}^{r}, and {c}${r} operators need
-    to be used (e.g. {c}@description ^cat{r}).
+    Except for the {c}ID{r}, {c}FILESAVED{r}, {c}USERUPDATE{r}, and {c}ACTIVE{r} fields, all search terms are searched
+    through the whole content of the various fields: i.e. {c}@description cat{r} will match any item whose description
+    field contains "cat". To match items that contain only "cat" (or start with, end with, etc.), the {c}%{r}, {c}_{r},
+    {c}^{r}, and {c}${r} operators need to be used (e.g. {c}@description ^cat{r}).
 
-    Search terms for {c}ID{r}, {c}AUTHOR{r}, and {c}USERNAME{r} are matched exactly as they are: i.e. {c}@author tom{r}
-    will match only items whose author field is exactly equal to "tom", to match items that contain "tom" the {c}%{r},
-    {c}_{r}, {c}^{r}, and {c}${r} operators need to be used (e.g. {c}@author %tom%{r}).
+    Search terms for {c}ID{r}, {c}FILESAVED{r}, {c}USERUPDATE{r}, and {c}ACTIVE{r} are matched exactly as they are: i.e.
+    {c}@id 1{r} will match only items whose ID field is exactly equal to "1", to match items that contain "1" the
+    {c}%{r}, {c}_{r}, {c}^{r}, or {c}${r} operators need to be used (e.g. {c}@id %1%{r}).
 
     For examples, please read the full README at {blue}https://pypi.org/project/{prog_name}/{version}{reset}.
     """
@@ -675,7 +680,7 @@ def database_search(ctx: Context, database: Callable[..., Database], table: str,
         results_total = sum(1 for _ in results.cursor)
 
     if show_sql:
-        echo(f"{blue}Query{reset}: {yellow}{sub(r'(?<=[()])[ ](?=[()])', '', query)}{reset}", color=ctx.color)
+        echo(f"{blue}Query{reset}: {yellow}{sub(r'(?<=[()]) (?=[()])', '', query)}{reset}", color=ctx.color)
         echo(f"{blue}Items{reset}: {yellow}{dumps(values)}{reset}", color=ctx.color)
     if total:
         echo(f"{blue}Total{reset}: {yellow}{results_total}{reset}", color=ctx.color)
@@ -691,6 +696,7 @@ def database_search(ctx: Context, database: Callable[..., Database], table: str,
 @color_option
 @help_option
 @pass_context
+@docstring_format()
 def database_view(ctx: Context, database: Callable[..., Database], table: str, id_: tuple[str | int, ...],
                   raw_content: bool, view_comments_: bool):
     """
@@ -709,7 +715,10 @@ def database_view(ctx: Context, database: Callable[..., Database], table: str, i
     if not (entry := get_table(db, table)[id_[0]]):
         secho(f"Entry {id_[0]!r} could not be found in {table.lower()}", fg="red", color=ctx.color)
     elif table == submissions_table:
-        echo(view_entry(entry, [SubmissionsColumns.DESCRIPTION.name], [], raw_html=raw_content), color=ctx.color)
+        fs, t = db.submissions.get_submission_files(id_[0])
+        echo(view_entry(
+            entry | {"FILES": list(map(str, filter(bool, [*(fs or []), t])))},
+            [SubmissionsColumns.DESCRIPTION.name], [], raw_html=raw_content), color=ctx.color)
         if view_comments_:
             echo(f"\n\n{blue}COMMENTS{reset}:")
             echo(view_comments(db.comments.get_comments_tree(submissions_table, id_[0]), raw_html=raw_content))
@@ -750,13 +759,13 @@ def database_remove(ctx: Context, database: Callable[..., Database], table: str,
         if id_ not in db_table:
             secho(f"Entry {id_!r} could not be found in {db_table.name.lower()}", fg="red", color=ctx.color)
         elif db_table.name.lower() == submissions_table.lower():
-            f, t = db.submissions.get_submission_files(id_)
+            fs, t = db.submissions.get_submission_files(id_)
             try:
                 del db_table[id_]
                 echo(f"Deleted entry {yellow}{id_}{reset} from {table}.", color=ctx.color)
             finally:
                 db.commit()
-                if f:
+                for f in fs or []:
                     f.unlink(missing_ok=True)
                 if t:
                     t.unlink(missing_ok=True)
@@ -767,136 +776,164 @@ def database_remove(ctx: Context, database: Callable[..., Database], table: str,
             finally:
                 db.commit()
 
+    if ids:
+        backup_database(db, ctx, "database")
+
 
 @database_app.command("add", short_help="Add entries manually.")
 @argument("table", nargs=1, required=True, is_eager=True, type=TableChoice())
 @argument("file", nargs=1, required=True, type=File("r"))
-@option("--submission-file", required=False, default=None, type=File("rb"))
-@option("--submission-thumbnail", required=False, default=None, type=File("rb"))
-@option("--replace", is_flag=True, default=False, show_default=True)
+@option("--submission-file", multiple=True, type=PathClick(exists=True, dir_okay=False, path_type=Path),
+        help="Specify a submission file.")
+@option("--submission-thumbnail", default=None, type=PathClick(exists=True, dir_okay=False, path_type=Path),
+        help="Specify a submission thumbnail.")
+@option("--replace", is_flag=True, default=False, show_default=True, help="Replace existing entries if present.")
 @database_exists_option
 @color_option
 @help_option
 @pass_context
-@docstring_format(prog_name=__prog_name__, version=__version__)
+@docstring_format(prog_name=__prog_name__, version=__version__, db_app=database_app.name)
 def database_add(ctx: Context, database: Callable[..., Database], table: str, file: TextIO,
-                 submission_file: BytesIO | None, submission_thumbnail: BytesIO | None, replace: bool):
+                 submission_file: tuple[Path], submission_thumbnail: Path | None, replace: bool):
     """
     Add entries and submission files manually using a JSON file. Submission files/thumbnails can be added using the
-    respective options.
+    respective options; all existing files are removed. Multiple submission files can be passed.
 
     The JSON file must contain fields for all columns of the table. For a list of columns for each table, please see
-    the README at {blue}https://pypi.org/project/{prog_name}/{version}{reset}.
+    the help of the {yellow}{db_app}{reset} command.
 
     By default, the program will throw an error when trying to add an entry that already exists. To override this
     behaviour and ignore existing entries, use the {yellow}--replace{reset} option.
     """
     db: Database = database()
+    db_table: Table = get_table(db, table)
+
+    data: dict = load(file)
+    file.close()
+    data = {k.upper(): v for k, v in data.items()}
+
+    if any(c.name.upper() not in data for c in db_table.columns):
+        raise BadParameter(f"Missing fields {set(map(str.upper, db_table.columns)) - set(data.keys())}"
+                           f" for table {table}",
+                           ctx, get_param(ctx, "file"))
+    elif not replace and (id_ := data[idc := db_table.key.name.upper()]) in db_table:
+        raise BadParameter(f"Entry with {idc} {id_!r} already exists in {table} table, but '--replace' is not set.",
+                           ctx, get_param(ctx, "file"))
 
     add_history(db, ctx, table=table, file=file.name, submission_file=submission_file is not None,
                 submission_thumbnail=submission_thumbnail is not None, replace=replace)
 
-    data: dict = load(file)
-    file.close()
-
-    data = {k.upper(): v for k, v in data.items()}
-
     if table.lower() == submissions_table.lower():
-        if any(c.name.upper() not in data for c in db.submissions.columns):
-            raise BadParameter(f"Missing fields {set(map(str.upper, db.submissions.columns)) - set(data.keys())}"
-                               f" for table {table}",
-                               ctx, get_param(ctx, "file"))
-        elif not replace and (id_ := data[idc := db.submissions.key.name.upper()]) in db.submissions:
-            raise BadParameter(f"Entry with {idc} {id_!r} already exists in {table} table, but '--replace' is not set.",
-                               ctx, get_param(ctx, "file"))
-        sub_file_orig, sub_thumb_orig = db.submissions.get_submission_files(data["id"])
-        sub_file: bytes | None = None
-        sub_thumb: bytes | None = None
-        if submission_file:
-            sub_file: bytes = submission_file.read()
-            submission_file.close()
-        elif sub_file_orig:
-            sub_file = sub_file_orig.read_bytes()
-        if submission_thumbnail:
-            sub_file: bytes = submission_thumbnail.read()
-            submission_thumbnail.close()
-        elif sub_thumb_orig:
-            sub_file = sub_thumb_orig.read_bytes()
+        sub_files_orig, sub_thumb_orig = db.submissions.get_submission_files(data["ID"])
+        if sub_files_orig:
+            for f in sub_files_orig:
+                f.unlink()
+        if sub_thumb_orig:
+            sub_thumb_orig.unlink()
+
+        sub_files: list[bytes] = [f.read_bytes() for f in submission_file]
+        sub_thumb: bytes | None = submission_thumbnail.read_bytes() if submission_thumbnail else None
+
         try:
-            db.submissions.save_submission(data, sub_file, sub_thumb, replace=replace)
+            db.submissions.save_submission(data, sub_files, sub_thumb, replace=replace)
         finally:
             db.commit()
     else:
-        db_table: Table = get_table(db, table)
-        if any(c.name.upper() not in data for c in db_table.columns):
-            raise BadParameter(f"Missing fields {set(map(str.upper, db_table.columns)) - set(data.keys())}"
-                               f" for table {table}",
-                               ctx, get_param(ctx, "file"))
-        elif not replace and (id_ := data[idc := db_table.key.name.upper()]) in db_table:
-            raise BadParameter(f"Entry with {idc} {id_!r} already exists in {table} table, but '--replace' is not set.",
-                               ctx, get_param(ctx, "file"))
         try:
             db_table.insert(db_table.format_entry(data), replace=replace)
         finally:
             db.commit()
 
+    backup_database(db, ctx, "database")
+
 
 @database_app.command("edit", short_help="Edit entries manually.")
 @argument("table", nargs=1, required=True, is_eager=True, type=TableChoice())
 @argument("_id", metavar="ID", nargs=1, required=True, is_eager=True)
-@argument("file", nargs=1, required=False, type=File("r"))
-@option("--submission-file", required=False, default=None, type=File("rb"))
-@option("--submission-thumbnail", required=False, default=None, type=File("rb"))
+@argument("file", nargs=1, required=False, default=None, type=File("r"))
+@option("--submission-file", multiple=True, type=PathClick(exists=True, dir_okay=False, path_type=Path),
+        help="Specify a submission file.")
+@option("--add-submission-files", is_flag=True, default=False, show_default=True,
+        help="Add submission files instead of replacing.")
+@option("--submission-thumbnail", default=None, type=PathClick(exists=True, dir_okay=False, path_type=Path),
+        help="Specify a submission thumbnail.")
 @database_exists_option
 @color_option
 @help_option
 @pass_context
-def database_edit(ctx: Context, database: Callable[..., Database], table: str, _id: str | int, file: TextIO,
-                  submission_file: BytesIO | None, submission_thumbnail: BytesIO | None):
+@docstring_format(prog_name=__prog_name__, version=__version__, db_app=database_app.name)
+def database_edit(ctx: Context, database: Callable[..., Database], table: str, _id: str | int, file: TextIO | None,
+                  submission_file: tuple[Path], add_submission_files: bool, submission_thumbnail: Path | None):
     """
     Edit entries and submission files manually using a JSON file. Submission files/thumbnails can be added using the
-    respective options.
+    respective options; existing files are overwritten unless the {yellow}--add-submission-files{reset} option is used.
+    Multiple submission files can be passed.
 
     The JSON fields must match the column names of the selected table. For a list of columns for each table, please see
-    the README at {blue}https://pypi.org/project/{prog_name}/{version}{reset}.
+    the help of the {yellow}{db_app}{reset} command.
 
     If the {yellow}--submission-file{reset} and/or {yellow}--submission-thumbnail{reset} options are used, the
     {yellow}FILE{reset} argument can be omitted.
     """
     db: Database = database()
     db_table: Table = get_table(db, table)
-    data: dict = load(file)
-    file.close()
+    data: dict = {}
+    if file:
+        data = load(file)
+        file.close()
 
     if not data and table.lower() != db.submissions.name.lower():
         raise BadParameter(f"FILE cannot be empty for {table} table.", ctx, get_param(ctx, "file"))
     elif not isinstance(data, dict):
         raise BadParameter(f"Data must be in JSON object format.", ctx, get_param(ctx, "file"))
+    elif submission_file and table.lower() != db.submissions.name.lower():
+        raise BadParameter(f"--submission-file option is not supported for {table}.", ctx,
+                           get_param(ctx, "submission_file"))
+    elif submission_thumbnail and table.lower() != db.submissions.name.lower():
+        raise BadParameter(f"--submission-thumbnail option is not supported for {table}.", ctx,
+                           get_param(ctx, "submission_file"))
+    elif _id not in db_table:
+        raise BadParameter(f"Cannot find {_id!r} in {table}.", ctx, get_param(ctx, "_id"))
 
-    add_history(db, ctx, table=table, id=_id, file=file.name,
-                submission_file=submission_file.name if submission_file else None,
+    add_history(db, ctx, table=table, id=_id, file=file.name if file else None,
+                submission_file=[f.name for f in submission_file] if submission_file else None,
                 submission_thumbnail=submission_thumbnail.name if submission_thumbnail else None)
 
     if (entry := db_table[_id]) is None:
         raise BadParameter(f"No entry with ID {_id} in {table}.", ctx, get_param(ctx, "_id"))
 
+    filesaved: int = data.get(f := SubmissionsColumns.FILESAVED.value.name, entry[f])
+
     if submission_file:
-        ext: str = db.submissions.save_submission_file(_id, submission_file.read(), "submission", "")
-        data = data | {(f := SubmissionsColumns.FILESAVED.value.name): (entry[f] & 0b01) + 0b10,
-                       SubmissionsColumns.FILEEXT.value.name: ext}
+        if not add_submission_files and entry[SubmissionsColumns.FILEEXT.value.name]:
+            fs, _ = db.submissions.get_submission_files(_id)
+            for f in fs:
+                f.unlink()
+        exts: list[str] = entry[SubmissionsColumns.FILEEXT.value.name] if add_submission_files else []
+        for n, f in enumerate(submission_file,
+                              len(entry[SubmissionsColumns.FILEEXT.value.name]) if add_submission_files else 0):
+            exts.append(db.submissions.save_submission_file(_id, f.read_bytes(), "submission", "", n))
+        filesaved = (filesaved & 0b001) + 0b110
+        data |= {SubmissionsColumns.FILESAVED.value.name: filesaved, SubmissionsColumns.FILEEXT.value.name: exts}
     if submission_thumbnail:
-        db.submissions.save_submission_file(_id, submission_file.read(), "submission", "")
-        data = data | {(f := SubmissionsColumns.FILESAVED.value.name): (entry[f] & 0b10) + 0b01}
+        db.submissions.save_submission_thumbnail(_id, submission_thumbnail.read_bytes())
+        filesaved = (filesaved & 0b100) + (filesaved & 0b010) + 0b001
+        data |= {SubmissionsColumns.FILESAVED.value.name: filesaved}
 
     if data:
-        db_table.update(Sb(db_table.key.name).__eq__(_id), data)
+        db_table.update(Sb(db_table.key.name).__eq__(_id), db_table.format_entry(data))
+        db.commit()
+
+    if submission_file or submission_thumbnail or data:
+        backup_database(db, ctx, "database")
 
 
 @database_app.command("clean", short_help="Clean database.")
 @database_exists_option
 @color_option
 @help_option
-def database_clean(database: Callable[..., Database]):
+@pass_context
+def database_clean(ctx: Context, database: Callable[..., Database]):
     """
     Clean database using the SQLite VACUUM function.
     """
@@ -907,6 +944,8 @@ def database_clean(database: Callable[..., Database]):
     db.commit()
     echo("Done")
 
+    backup_database(db, ctx, "database")
+
 
 # noinspection DuplicatedCode
 @database_app.command("copy", short_help="Copy database entries.")
@@ -997,6 +1036,8 @@ def database_merge(ctx: Context, database: Callable[..., Database], database_ori
 
     echo("Done")
 
+    backup_database(db, ctx, "database")
+
 
 @database_app.command("upgrade", short_help="Upgrade database.")
 @database_exists_option
@@ -1024,6 +1065,7 @@ database_app.list_commands = lambda *_: [
     database_search.name,
     database_view.name,
     database_add.name,
+    database_edit.name,
     database_remove.name,
     database_merge.name,
     database_copy.name,
