@@ -23,6 +23,7 @@ from .colors import *
 from .util import CompleteChoice
 from .util import CustomHelpColorsGroup
 from .util import add_history
+from .util import backup_database
 from .util import color_option
 from .util import database_exists_option
 from .util import docstring_format
@@ -176,10 +177,13 @@ def download_users(ctx: Context, database: Callable[..., Database], users: tuple
     folders = folders_
     try:
         downloader.download_users(list(users), list(folders))
+        backup_database(db, ctx, "download")
     except Unauthorized as err:
         secho(f"\nError: Unauthorized{(': ' + ' '.join(err.args)) if err.args else ''}", fg="red", color=ctx.color)
+        backup_database(db, ctx, "download")
     except RequestException as err:
         secho(f"\nError: An error occurred during download: {err!r}.", fg="red", color=ctx.color)
+        backup_database(db, ctx, "download")
     finally:
         echo()
         downloader.verbose_report() if verbose_report else downloader.report()
@@ -242,10 +246,13 @@ def download_update(ctx: Context, database: Callable[..., Database], users: tupl
         add_history(db, ctx, users=users, folders=folders, stop=stop)
     try:
         downloader.download_users_update(list(users), list(folders), stop, deactivated, like)
+        backup_database(db, ctx, "download")
     except Unauthorized as err:
         secho(f"\nError: Unauthorized{(': ' + ' '.join(err.args)) if err.args else ''}", fg="red", color=ctx.color)
+        backup_database(db, ctx, "download")
     except RequestException as err:
         secho(f"\nError: An error occurred during download: {err!r}.", fg="red", color=ctx.color)
+        backup_database(db, ctx, "download")
     finally:
         echo()
         downloader.verbose_report() if verbose_report else downloader.report()
@@ -259,7 +266,7 @@ def download_update(ctx: Context, database: Callable[..., Database], users: tupl
           callback=lambda _c, _p, v: sorted(set(v), key=v.index))
 @option("--replace", is_flag=True, default=False, show_default=True, help="Replace submissions already in database.")
 @retry_option
-@comments_option
+@option("--save-comments", is_flag=True, default=False, help="Save submissions' comments.")
 @dry_run_option
 @verbose_report_option
 @report_file_option
@@ -292,10 +299,13 @@ def download_submissions(ctx: Context, database: Callable[..., Database], submis
         add_history(db, ctx, submission_id=submission_id, replace=replace)
     try:
         downloader.download_submissions(list(submission_id))
+        backup_database(db, ctx, "download")
     except Unauthorized as err:
         secho(f"\nError: Unauthorized{(': ' + ' '.join(err.args)) if err.args else ''}", fg="red", color=ctx.color)
+        backup_database(db, ctx, "download")
     except RequestException as err:
         secho(f"\nError: An error occurred during download: {err!r}.", fg="red", color=ctx.color)
+        backup_database(db, ctx, "download")
     finally:
         echo()
         downloader.verbose_report() if verbose_report else downloader.report()
@@ -308,7 +318,7 @@ def download_submissions(ctx: Context, database: Callable[..., Database], submis
 @argument("journal_id", nargs=-1, required=True, type=IntRange(1),
           callback=lambda _c, _p, v: sorted(set(v), key=v.index))
 @option("--replace", is_flag=True, default=False, show_default=True, help="Replace journals already in database.")
-@comments_option
+@option("--save-comments", is_flag=True, default=False, help="Save journals' comments.")
 @dry_run_option
 @verbose_report_option
 @report_file_option
@@ -338,10 +348,13 @@ def download_journals(ctx: Context, database: Callable[..., Database], journal_i
         add_history(db, ctx, journal_id=journal_id, replace=replace)
     try:
         downloader.download_journals(list(journal_id))
+        backup_database(db, ctx, "download")
     except Unauthorized as err:
         secho(f"\nError: Unauthorized{(': ' + ' '.join(err.args)) if err.args else ''}", fg="red", color=ctx.color)
+        backup_database(db, ctx, "download")
     except RequestException as err:
         secho(f"\nError: An error occurred during download: {err!r}.", fg="red", color=ctx.color)
+        backup_database(db, ctx, "download")
     finally:
         echo()
         downloader.verbose_report() if verbose_report else downloader.report()
